#version 430 core

layout (triangles) in;
layout (triangle_strip, max_vertices = 18) out;

// Input variables
in VertexData
{
	vec2 tex_coord;
} inData[3];

// Output variables
out VertexData
{
	vec3 tex_coord;
} outData;

// Data of all cameras
struct SingleCameraData
{
	mat4 projection;		// Projection matrix
	mat4 projection_inv;	// Inverse of the projection matrix
	mat4 view;				// View matrix
	mat4 view_inv;			// Inverse of the view matrix
	mat3 view_it;			// Inverse of the transpose of the top-left part 3x3 of the view matrix
	vec3 eye_position;		// Position of the eye in world space
};
layout (std140, binding = 0) uniform CubeCameraData
{
	SingleCameraData cameras[6];
};

//-----------------------------------------------------------------------

void main()
{
	// Task 2 (optional): Implement rendering into the cubemap
	for (int layer = 0; layer < 6; layer++)
	{
		for (int v = 0; v < 3; v++)
		{
			// Compute the texture coordinate, using the proper camera
			vec4 pos_cs = vec4(inData[v].tex_coord * 2.0 - 1.0, 0.0, 1.0);
			vec4 pos_vs = cameras[layer].projection_inv * pos_cs;
			vec3 dir_vs = pos_vs.xyz;
			vec3 dir_ws = mat3(cameras[layer].view_inv) * dir_vs;
			outData.tex_coord = dir_ws;
			
			// Compute the gl_Position and gl_Layer
			gl_Position = pos_cs;
			gl_Layer = layer;
			
			// Emit the vertex
			EmitVertex();
		}
		EndPrimitive();
	}
}
