#version 430 core

#extension GL_ARB_shader_viewport_layer_array : require

const vec2 tex_coords[3] = vec2[3] (
	vec2(0.0, 0.0),
	vec2(2.0, 0.0),
	vec2(0.0, 2.0)
);

out VertexData
{
	vec3 tex_coord;
} outData;

// Data of all cameras
struct SingleCameraData
{
	mat4 projection;		// Projection matrix
	mat4 projection_inv;	// Inverse of the projection matrix
	mat4 view;				// View matrix
	mat4 view_inv;			// Inverse of the view matrix
	mat3 view_it;			// Inverse of the transpose of the top-left part 3x3 of the view matrix
	vec3 eye_position;		// Position of the eye in world space
};
layout (std140, binding = 0) uniform CubeCameraData
{
	SingleCameraData cameras[6];
};

void main()
{
	// Task 5 (optional): Compute all data here, including gl_Layer, use the index of the current instance (gl_InstanceID)

	// gl_VertexID holds the index of the vertex that is being processed

	// Compute the texture coordinate, using the proper camera
	vec4 pos_cs = vec4(tex_coords[gl_VertexID] * 2.0 - 1.0, 0.0, 1.0);
	vec4 pos_vs = cameras[gl_InstanceID].projection_inv * pos_cs;
	vec3 dir_vs = pos_vs.xyz;
	vec3 dir_ws = mat3(cameras[gl_InstanceID].view_inv) * dir_vs.xyz;
	outData.tex_coord = dir_ws;
	
	// Compute the gl_Position and gl_Layer
	gl_Position = pos_cs;
	gl_Layer = gl_InstanceID;
}
