#version 430 core

// Input variables
layout (location = 0) in vec4 position;
layout (location = 1) in vec3 normal;
layout (location = 2) in vec2 tex_coord;

// Output variables
out VertexData
{
	vec3 normal_ws;		// Normal in world space
	vec3 position_ws;	// Position in world space
	vec2 tex_coord;
	int layer;
} outData;

// Data of the camera
struct SingleCameraData
{
	mat4 projection;		// Projection matrix
	mat4 projection_inv;	// Inverse of the projection matrix
	mat4 view;				// View matrix
	mat4 view_inv;			// Inverse of the view matrix
	mat3 view_it;			// Inverse of the transpose of the top-left part 3x3 of the view matrix
	vec3 eye_position;		// Position of the eye in world space
};
layout (std140, binding = 0) uniform CubeCameraData
{
	SingleCameraData cameras[6];
};

// Data of the object
layout (std140, binding = 2) uniform ModelData
{
	mat4 model;			// Model matrix
	mat4 model_inv;		// Inverse of the model matrix
	mat3 model_it;		// Inverse of the transpose of the top-left part 3x3 of the model matrix
};

//-----------------------------------------------------------------------

void main()
{
	// Compute the data of the vertex
	outData.tex_coord = tex_coord;
	outData.position_ws = vec3(model * position);
	outData.normal_ws = normalize(model_it * normal);

	// Task 4: Compute gl_Position and outData.layer, based on the index of the current instance (gl_InstanceID)

	// Pass the layer from gl_InstanceID
	outData.layer = gl_InstanceID;

	// Compute the gl_Position (using the proper camera)
	gl_Position = cameras[gl_InstanceID].projection * cameras[gl_InstanceID].view * model * position;
}
